import { promisify } from 'util';
import { exec } from 'child_process';

const exec_async = promisify(exec);

export const typstPreprocess = (config) => {
  const {
	extensions,
  } = config;

  return {
    name: "typst",
    markup: async ({content: _, filename}) => {

      const extensionsParts = extensions.map((ext) =>
			  ext.startsWith('.') ? ext : '.' + ext
		  );
		  if (!extensionsParts.some((ext) => filename.endsWith(ext))) return;

      const result_metadata = await exec_async(`typst query --features html ${filename} "<post>" --field value --one`);
      const metadata = String(result_metadata.stdout);

      const result_post = await exec_async(`typst compile --features html --format html ${filename} -`);
      // strip the body tag
      const bodyHTML = /<body.*?>([\s\S]*)<\/body>/.exec(result_post.stdout)[1]
      const code = `<script context='module'>\n export const metadata = ${metadata}\n</script>` + bodyHTML

      return {
        code,
        data: {
          fm: metadata,
        },
      }
    }
  };
}
